
#include "rpds_software.h"

/* Interrupt handler for the pushbuttons. */
static void buttons_isr( void* context, alt_u32 id )
{
  volatile int *function = (volatile int*) context;
  
  *function = IORD_ALTERA_AVALON_PIO_EDGE_CAP( BUTTONS_BASE );
  IOWR_ALTERA_AVALON_PIO_EDGE_CAP( BUTTONS_BASE, 0 );
  IOWR_ALTERA_AVALON_PIO_IRQ_MASK( BUTTONS_BASE, 0xF );
}

//  LCD Module 16*2
#define lcd_write_cmd(base, data)                     IOWR(base, 0, data)
#define lcd_read_cmd(base)                            IORD(base, 1)
#define lcd_write_data(base, data)                    IOWR(base, 2, data)
#define lcd_read_data(base)                           IORD(base, 3) 

void LCD_Show_Text(char* Text)
{
  int i;
  for(i=0;i<strlen(Text);i++)
  {
    lcd_write_data(LCD_BASE,Text[i]);
    usleep(2000);
  }
}
//-------------------------------------------------------------------------
void LCD_Line2()
{
  lcd_write_cmd(LCD_BASE,0xC0);
  usleep(2000);
}

static void lcd_init()
{
  /* Set Function Code Four Times -- 8-bit, 2 line, 5x7 mode */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x38 );
  usleep(4100);   /* Wait 4.1 ms */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x38 );
  usleep(100);    /* Wait 100 us */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x38 );
  usleep(5000);   /* Wait 5.0 ms */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x38 );
  usleep(100);   /* Wait 100 us */
  /* Set Display to OFF */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x08 );
  usleep(100);   /* Wait 100 us */
  /* Set Display to ON */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x0C );
  usleep(100);   /* Wait 100 us */
  /* Set Entry Mode -- Cursor increment, display doesn't shift */  
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x06 );
  usleep(100);   /* Wait 100 us */
  /* Set the cursor to the home position */  
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x02 );
  usleep(2000);   /* Wait 2 ms */
  /* Clear the display */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x01 );
  usleep(2000);   /* Wait 2 ms */
}

alt_u32 test_lcd()
{
  int i;
  char message[17] = "Counting...     ";
  char done[12] = "Done!      ";
  
  /* Write a simple message on the first line. */
  for( i = 0; i < 16; i++ )
  {
    IOWR( LCD_BASE, LCD_WR_DATA_REG, message[i] );
    usleep(100);
  }
  
  /* Count along the bottom row */

  /* Set Address */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0xC0 );
  usleep(1000);
  
  /* Display Count */
  for( i = 0; i < 10; i++ )
  {
    IOWR( LCD_BASE, LCD_WR_DATA_REG, (char)(i+0x30) );
    usleep(500000);   /* Wait 0.5 sec. */
  }

 /* Write "Done!" message on first line. */

 /* Set Address */
  IOWR( LCD_BASE, LCD_WR_COMMAND_REG, 0x80 );
  usleep(1000);
  
  /* Write data */
  for( i = 0; i < 11; i++ )
  {
    IOWR( LCD_BASE, LCD_WR_DATA_REG, done[i] );
    usleep(100);
  }
  return(0);
}

alt_u32 test_sram2()
{
  alt_u32 i;
  alt_u32 errors = 0;
  alt_u32 *buffer = (alt_u32 *)SRAM_BASE;

  /* Write data to SRAM */
  printf( "   Writing Data\n" );
  for( i = 0; i < SRAM_MAX_WORDS; i++ )
  {
    buffer[i] = (i + 1000000);
  }

  /* Check output from SRAM Memory */
  printf( "   Reading Data\n" );
  for( i = 0; i < SRAM_MAX_WORDS; i++ )
  {
    if( buffer[i] != (i+1000000) )
    {
      errors++;
    }      
  }
  return( errors );
}
alt_u32 test_sram()
{
  alt_u32 i;
  alt_u32 errors = 0;
  alt_u32 buffer[SRAM_MAX_WORDS];

  /* Write data to SRAM */
  for( i = 0; i < SRAM_MAX_WORDS; i++ ) {
    buffer[i] = i + 0x70000000;
  }
  /* Check output from SRAM Memory */
  for( i = 0; i < SRAM_MAX_WORDS; i++ ) {
    if( buffer[i] != (i+0x70000000) )
      errors++;
  }
  return( errors );
}

alt_u32 test_flash()
{
  alt_u32 i;
  alt_u32 in_buff[FLASH_MAX_WORDS];
  alt_u32 out_buff[FLASH_MAX_WORDS];
  int errors = 0;
  alt_flash_fd* flash_handle;
  
  flash_handle = alt_flash_open_dev( FLASH_NAME );

  /* Create data buffer to write to Flash */
  for( i = 0; i < FLASH_MAX_WORDS; i++ )
  {
    in_buff[i] = i + 1000000;
  }

  /* Write data to Flash memory */
  alt_write_flash( flash_handle, 0, (char *)in_buff, FLASH_MAX_WORDS*4 );
  printf("   Writing done\n");
  usleep(50000);
  
  /* Read data from Flash memory */
   alt_read_flash( flash_handle, 0, out_buff, FLASH_MAX_WORDS*4 );
  printf("   Reading done\n");
  usleep(50000);

  /* Check output from Flash Memory */
  for( i = 0; i < FLASH_MAX_WORDS; i++ )
  {
    if( out_buff[i] != (i+1000000) )
      errors++;
  }
  
  alt_flash_close_dev( flash_handle );
  return( errors );
}

alt_u32 test_sdram2()
{
  alt_u32 i;
  alt_u32 errors = 0;
  alt_u32 buffer[SDRAM_MAX_WORDS];

  /* Write data to SRAM */
  for( i = 0; i < SDRAM_MAX_WORDS; i++ ) {
    buffer[i] = i + 0x50000000;
  }
  /* Check output from SRAM Memory */
  for( i = 0; i < SDRAM_MAX_WORDS; i++ ) {
    if( buffer[i] != (i+0x50000000) )
      errors++;
  }
  return( errors );
}
alt_u32 test_sdram()
{
  alt_u32 i;
  alt_u32 errors = 0;
  alt_u32 *buffer = (alt_u32 *)SDRAM_BASE;

  /* Write data to SDRAM */
  printf( "   Writing Data\n" );
  for( i = 0; i < SDRAM_MAX_WORDS; i++ )
  {
    buffer[i] = (i + 1000000);
  }

  /* Check output from SDRAM Memory */
  printf( "   Reading Data\n" );
  for( i = 0; i < SDRAM_MAX_WORDS; i++ )
  {
    if( buffer[i] != (i+1000000) )
    {
      errors++;
    }      
  }
  return( errors );
}

int main( void )
{
  volatile int function = 0;
  alt_u32 switches, ret_val;
  alt_u16 count = 0;
  alt_u32 counttemp;
  
  printf( "Welcome to the Nios II Reference Design Test Program\n" );


  IOWR_ALTERA_AVALON_PIO_EDGE_CAP( BUTTONS_BASE, 0 );  
  alt_irq_register( BUTTONS_IRQ, (void *) &function, buttons_isr ); 
  IOWR_ALTERA_AVALON_PIO_IRQ_MASK( BUTTONS_BASE, 0xF );

  while(1)
  {
    switch( function )
    {
      case 0x1:           /* Test the LCD Display */
        printf( "Testing LCD Display\n" );
        lcd_init();
        ret_val = test_lcd();
        printf( "...Completed.\n" );
        break;
      case 0x2:           /* Test the SRAM Memory */
        printf("Testing SRAM Memory\n" );
        /* NOTE: SDRAM is used as program memory for the DE2 board    *
         *       instead of SRAM. test_sram2() and test_sdram2() have *
         *       been written to take this difference into account.   */
        ret_val = test_sram2();
        printf("...Completed with %d Errors.\n", (int)ret_val );
        break;
      case 0x4:           /* Test the Flash Memory */
        printf("Testing Flash Memory\n" );
        ret_val = test_flash();
        printf("...Completed with %d Errors.\n", (int)ret_val );
        break;
      case 0x8:           /* Test the SDRAM Memory */
        printf("Testing SDRAM Memory\n" );
        /* NOTE: SDRAM is used as program memory for the DE2 board    *
         *       instead of SRAM. test_sram2() and test_sdram2() have *
         *       been written to take this difference into account.   */
        ret_val = test_sdram2();
        printf("...Completed with %d Errors.\n", (int)ret_val );
        break;
      default:            /* Do nothing */
        break;
    }

    function = 0;

    switches = IORD_ALTERA_AVALON_PIO_DATA( SWITCHES_BASE );
    IOWR_ALTERA_AVALON_PIO_DATA( LED_RED_BASE, switches );
    IOWR_ALTERA_AVALON_PIO_DATA( LED_GREEN_BASE, (switches&0x1ff) );

    counttemp = (count<<16) | count;
    IOWR( SEVEN_SEG_BASE, 0, counttemp );
    count++;
 
    usleep( 50000 );
  }

  return(0);  
}
